/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {

dojo.provide("com.ibm.rational.report.web.core.app.view.report.EditReportNodeViewer");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Toolbar");
dojo.require("dijit.form.ComboBox");
dojo.require("dijit.form.Button");
dojo.require("dijit.layout.LayoutContainer");
dojo.require("dijit.layout.ContentPane");
dojo.require("dijit.layout.TabContainer");
dojo.require("dojo.parser");
dojo.require("com.ibm.rational.report.web.core.app.internal.MessageFormatter");
dojo.require("com.ibm.rational.report.web.core.app.internal.common.ErrorDialog");
dojo.require("com.ibm.rational.report.web.core.app.internal.common.PermissionsWidget");
dojo.require("com.ibm.rational.report.web.core.app.internal.Util");
dojo.require("com.ibm.rational.report.web.core.client.internal.ReportClient");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.rational.report.web.core.app.view.report.ReportContent");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.internal.common", "Messages");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.serviceability", "Messages");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.view.report", "Messages");

var CommonMessages	    	= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.internal.common", "Messages");
var ServiceabilityMessages	= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.serviceability", "Messages");
var ReportMessages			= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.view.report", "Messages");
var ReportClient			= com.ibm.rational.report.web.core.client.internal.ReportClient;
var MessageFormatter		= com.ibm.rational.report.web.core.app.internal.MessageFormatter;
var Util					= com.ibm.rational.report.web.core.app.internal.Util;
var session					= com.ibm.team.repository.web.client.session;
var ServiceResponseHandler 	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var PermissionsWidget 		= com.ibm.rational.report.web.core.app.internal.common.PermissionsWidget;
var ReportContent			= com.ibm.rational.report.web.core.app.view.report.ReportContent;
var TeamServerClient 		= com.ibm.team.repository.web.transport.TeamServerClient;

var REPORTS_ERROR_DIALOG    = "com.ibm.rational.report.web.core.app.view.admin.ReportsErrorDialog";

var SERVICE_URI				= "com.ibm.rational.report.common.IReportContentRestService/";
var ACTION_PROP				= "__upLoadReportDesignActionProp";

var REPORT_DESIGN_IMG_SRC			= dojo.moduleUrl("com.ibm.rational.report.web.core", "app/internal/common/icons/report_design.gif");
var REPORT_DESIGN_SHARED_IMG_SRC	= dojo.moduleUrl("com.ibm.rational.report.web.core", "app/internal/common/icons/report_design_shared.gif");
var REPORT_QUERY_IMG_SRC			= dojo.moduleUrl("com.ibm.rational.report.web.core", "app/internal/common/icons/report_query.gif");
var REPORT_QUERY_SHARED_IMG_SRC		= dojo.moduleUrl("com.ibm.rational.report.web.core", "app/internal/common/icons/report_query_shared.gif");
var REPORT_SNAPSHOT_IMG_SRC			= dojo.moduleUrl("com.ibm.rational.report.web.core", "app/internal/common/icons/report_snapshot.gif");
var REPORT_SNAPSHOT_SHARED_IMG_SRC	= dojo.moduleUrl("com.ibm.rational.report.web.core", "app/internal/common/icons/report_snapshot_shared.gif");

/**
 * Displays the Edit Report Folder/Node view.
 */
dojo.declare(
	"com.ibm.rational.report.web.core.app.view.report.EditReportNodeViewer",
	[dijit._Widget, dijit._Templated],	{
	
	templatePath: dojo.moduleUrl("com.ibm.rational.report.web.core", "app/view/report/templates/EditReportNodeViewer.html"),
	
	startup: function() {
		this.inherited(arguments);
		// instantiate widgets used for tabs
		var nodeList = [];
		nodeList.push(this.layoutContainer);
		nodeList.push(this.mainContentPane);
		nodeList.push(this.tabContainer);
		nodeList.push(this.detailsContentPane);
		if (this._createPermissionsTab) {
			nodeList.push(this.permissionContentPane);
		}
		nodeList.push(this.viewReportContentPane);
		var nodes = dojo.parser.instantiate(nodeList);
		
		var tabContainerId = nodes[2].id;
		dojo.subscribe(tabContainerId+"-selectChild", this, "_newTabSelected");
		this.nodeShared.checked = this._selectedReport.shared;	//fix for IE6
	},
	
	constructor: function(args) {
		this._controller = args.controller;
		this._selectedNode = args.node;
		this._selectedReport = args.selectedReport;
		this._createPermissionsTab = true;
		if (args.createPermissionsTab != null) {
			this._createPermissionsTab = args.createPermissionsTab;
		}
		this._args = args;
	},
	
	postCreate: function() {
		this._initText();
		
		this._msgFormatter = new MessageFormatter(this.messageArea);
		
		this.szReportFiles = null;
		this.masterDesignFile = null;
						
		this.count = 1;
		this.fileListRowId = 0;
		this.currentInput = this.reportDesignFileInput;
		this._fileListArray = [];
	
		this.szReportDesigns=null;
		this.szReportDesign=null;
		this.updateReportDesign=false;
		
		this._postServiceExec = null;
		this._postSave = null;
		this._postCancel = null;
		
		if(this._selectedNode != null) {
			this._selectedNodeId = this._selectedNode.itemId;
			this._resourceType = this._selectedNode.nodeType;
		}
		else {
			this._resourceType = this._selectedReport.type;
		}
		
		this._readOnly = false;
		this._isOwner = false;
		this._isProjectLead = false;
		
		var args = this._args;
		
		if (typeof args.postSave != "undefined") {
			this._postSave = args.postSave;
		}
		
		if (typeof args.postCancel != "undefined") {
			this._postCancel = args.postCancel;
		}
		
		if (typeof this._selectedNodeId == "undefined") {
			this._selectedNodeId = null;
		}

		this._getSelectedNode();		
		
		this._fillParentNodeControl();
		this._fillOtherControls();
		
		if (this._resourceType == "REPORT_QUERY" || this._resourceType == "REPORT_SNAPSHOT") {
			this._hideControls();
		}
				
		if (this._selectedReport != null) {
			this.repositoryItemId = this._selectedReport.itemId;
		}
		this.permissionsWidget = null;
		this.reportContent = null;
		
		var self = this;
		
		this._parentFolderChanged = false;
		this._nameChanged = false;
		this._descriptionChanged = false;
		this._nodeSharedChanged = false;
		this._reportDesignFilesChanged = false;
		this._masterDesignFileChanged = false;

		// save and cancel actions
		dojo.connect(this.saveButton, "onclick",
			function(e) {
				e.preventDefault();
				self._handleSave();
				self._parentFolderChanged = false;
				self._nameChanged = false;
				self._descriptionChanged = false;
				self._nodeSharedChanged = false;
				self._reportDesignFilesChanged = false;
			}
		);
		
		dojo.connect(this.closeButton, "onclick",
			function(e) {
				e.preventDefault();
				if (self._postCancelFun) {
					self._postCancelFun();
				}
				else {
					self._controller.defaultAction();
				}
			}
		);
		
		this._createTeamAreaCombo();
		
		var postSave = function() {
			self._controller.editReportNode(self._selectedNode,
				self._selectedReport, self._postSave, self._postCancel);
		};
		
		var postCancel = function() {
			self._controller.editReportNode(self._selectedNode,
				self._selectedReport, self._postSave, self._postCancel);
		};
		
		dojo.connect(this.parentFolder, "onchange",
			function(e) {
				e.preventDefault();
				if (self._selectedNode) {
					self._handleParentChange();					
					var parentId = (self._selectedNode.parent != null) ? self._selectedNode.parent.itemId : "";
					self._parentFolderChanged = (parentId != self.parentFolder.options[self.parentFolder.selectedIndex].value);
					self._updateSaveButtonState();
				}
			}
		);
		
		dojo.connect(this.nodeName, "onkeyup",
			function(e) {
				e.preventDefault();
				var name = dojo.trim(self.nodeName.value);
				self._nameChanged = (name != self._selectedReport.name) ;
				self._updateSaveButtonState(); 
				if(name.length==0) {
					self.saveButton.disabled=true;
				}
			}
		);
		
		dojo.connect(this.nodeName, "onkeypress",
			function(e) {
				if(e.charCode == 47 || e.charCode == 92 || e.charCode == 42 || e.charCode == 63 ||
				   e.charCode == 124 || e.charCode == 62 || e.charCode == 60 || e.charCode == 58  || e.charCode == 34){
						e.preventDefault();
				}
			}
		);
		
		dojo.connect(this.description, "onkeyup",
			function(e) {
				e.preventDefault();
				var description = dojo.trim(self.description.value);
				self._descriptionChanged = (description != self._selectedReport.description) ;
				self._updateSaveButtonState();
			}
		);
		
		dojo.connect(this.nodeShared,"onclick",
			function(e){
				self._nodeSharedChanged = (self.nodeShared.checked != self._selectedReport.shared);
				self._updateSaveButtonState();
			}
		);
		
		dojo.connect(this.reportDesignFileInput, "onchange", this, "_addFileInput");
		
		dojo.connect(this._reportDesignFilesTable,"onclick", function(e){
			var target;
			if(e.target) {
				target = e.target;
			} else if(e.srcElement) {
				target = e.srcElement;
			}
			if(target.type == "radio") {
				self._masterDesignFileChanged = (self.masterDesignFile != target.id);
				self._updateSaveButtonState();
			}
		});
			
		this._updateSaveButtonState();
				
		// initialize the report design files table
		var chk = document.createElement("input");
		chk.type = "checkbox";
		
		this._tableColumns = [
			{field: "Select", sortType:"__markup__", label:chk,  width: "12px"},
			{field: "FileName", label: CommonMessages.FILENAME},
			{field: "Master", label: CommonMessages.MASTER, sortType: "__markup__",noSort: true}			
		];
		
		var toolbar = new dijit.Toolbar({
				layoutAlign: "top",
				style: "float: left; background : transparent; border: none"
		});
		
		this.downloadButton = new dijit.form.Button({
            title: CommonMessages.DOWNLOAD_REPORTS_TOOLTIP,
            iconClass: "DownloadButtonIcon",
			label: CommonMessages.DOWNLOAD_REPORTS_TOOLTIP,
			showLabel: false,
            onClick: dojo.hitch(this, this._downloadReportDesignFiles)
        });
        
		this.deleteButton = new dijit.form.Button({
            title: CommonMessages.DELETE_REPORTS_TOOLTIP,
            iconClass: "DeleteButtonIcon",
			label: CommonMessages.DELETE_REPORTS_TOOLTIP,
			showLabel: false,
            onClick: dojo.hitch(this, this._deleteReportDesignFiles)
        });
		
		this.downloadButton.disabled = true;
		this.deleteButton.disabled = true;
		this.downloadButton._setStateClass();
		this.deleteButton._setStateClass();
		
		toolbar.addChild(this.downloadButton); 
		toolbar.addChild(this.deleteButton);
		
		this._tableColumns.push({
				field: "Toolbar",
				sortType: "__markup__",
				label: toolbar.domNode
		});
		
				
		this._table = new com.ibm.rational.report.web.core.app.internal.common.FilteringTable(
				{valueField: "rowId", noSelectField: "Select"}, this._reportDesignFilesTable);
		
		for (var i = 0; i < this._tableColumns.length; i++) {
			this._table.columns.push(this._table.createMetaData(this._tableColumns[i]));
		}
		
		dojo.connect(this._table, "onGroupSelect", this, "_onGroupSelect");
		
		if (this._resourceType == "REPORT_DESIGN") {
			var deferred = this._getReportDesignFiles();
			deferred.addCallback(dojo.hitch(this,"reportDesignFilesHandler"));
		}
		if (this._createPermissionsTab == false) {
			this.tabContainer.removeChild(this.permissionContentPane);
		}
		
		dojo.connect(this.reportDesignFileInput, "onkeypress", 
			function(e){				
				if(e.keyCode == dojo.keys.TAB || e.keyCode == dojo.keys.ENTER){
					return;
					//do nothing
				}else{//else prevent anything from being entered
					e.preventDefault();	
					return false;
				}
		});		
	},
	
	_getSelectedNode: function() {
	 	if (this._selectedNodeId != null) {
			var widget = dijit.byId(this._selectedNodeId);
			if(widget!=null) {
				this._selectedNode = widget._report;
			}
			if (this._selectedNode != null) {
				this._isOwner = this._controller.secMgr.isOwner(this._selectedNode);
				this._isProjectLead = this._controller.secMgr.isProjectLead(this._selectedNode);
			}
		}		
	 },
	 
	 _refreshSelectedReport: function() {
		var name = dojo.trim(this.nodeName.value);
		this._selectedReport.name = name;
		var description = dojo.trim(this.description.value);
		this._selectedReport.description = description;		 	
		this._selectedReport.shared = this.nodeShared.checked;
	 },
	 
	_initText: function() {
		Util.removeChildren(this.saveButton);
		this.saveButton.appendChild(document.createTextNode(CommonMessages.SAVE_BUTTON_LABEL));
		
		Util.removeChildren(this.closeButton);
		this.closeButton.appendChild(document.createTextNode(CommonMessages.CLOSE_BUTTON_LABEL));
		
		this.detailsContentPane.title = ReportMessages.EDIT_REPORT_NODE_VIEWER_DETAILS_TAB_TITLE;
		this.permissionContentPane.title = ReportMessages.EDIT_REPORT_NODE_VIEWER_PERMISSIONS_TAB_TITLE;
		this.viewReportContentPane.title = ReportMessages.EDIT_REPORT_NODE_VIEWER_REPORT_TAB_TITLE;	
		
		Util.removeChildren(this.nodeNameLabel);
		this.nodeNameLabel.appendChild(document.createTextNode(ReportMessages.EDIT_REPORT_NODE_VIEWER_NAME_LABEL));
		
		Util.removeChildren(this.descriptionLabel);
		this.descriptionLabel.appendChild(document.createTextNode(ReportMessages.EDIT_REPORT_NODE_VIEWER_DESCRIPTION_LABEL));
		
		Util.removeChildren(this.typeLabel);
		this.typeLabel.appendChild(document.createTextNode(ReportMessages.EDIT_REPORT_NODE_VIEWER_TYPE_LABEL));
		
		Util.removeChildren(this.parentFolderLabel);
		this.parentFolderLabel.appendChild(document.createTextNode(ReportMessages.EDIT_REPORT_NODE_VIEWER_PARENT_FOLDER_LABEL));
		
		Util.removeChildren(this.nodeSharedLabel);
		this.nodeSharedLabel.appendChild(document.createTextNode(ReportMessages.EDIT_REPORT_NODE_VIEWER_NODE_SHARED_LABEL));
		
		Util.removeChildren(this.fileLabel);
		this.fileLabel.appendChild(document.createTextNode(ReportMessages.NEW_REPORT_DESIGN_VIEWER_FILE_LABEL));
		
		Util.removeChildren(this.filePromptLabel);
		this.filePromptLabel.appendChild(document.createTextNode(ReportMessages.NEW_REPORT_DESIGN_VIEWER_FILE_PROMPT_LABEL));
	},
		
	_setPageTitle: function(title) {
		Util.removeChildren(this.pageTitle);
		var pageTitleIcon = document.createElement("img");
		if (this._resourceType == "REPORT_QUERY") {
			pageTitleIcon.src = this._selectedReport.shared ? REPORT_QUERY_SHARED_IMG_SRC : REPORT_QUERY_IMG_SRC;
		}
		else if (this._resourceType == "REPORT_SNAPSHOT") {
		   	pageTitleIcon.src = this._selectedReport.shared ? REPORT_SNAPSHOT_SHARED_IMG_SRC : REPORT_SNAPSHOT_IMG_SRC;
		}
		else {
			pageTitleIcon.src = this._selectedReport.shared ? REPORT_DESIGN_SHARED_IMG_SRC : REPORT_DESIGN_IMG_SRC;
		}
		this.pageTitle.appendChild(pageTitleIcon);
		var pageTitleText = document.createTextNode(title.replace(" ", "\xa0", "g")); // preserve the spaces
		this.pageTitle.appendChild(pageTitleText);
	},
		
	_fillParentNodeControl:  function() {
		if (this._selectedNodeId) {
			var selectedNode = this._selectedNode;
			
			if (!(selectedNode != null && selectedNode.report)) {
				this.parentFolder.options[0] = new Option(CommonMessages.NO_PARENT_FOLDER_OPTION, "", false, false);
			}
			
			var selectedNodeParentNodeId = null;
			if (selectedNode != null && selectedNode.parent) {
				selectedNodeParentNodeId = selectedNode.parent.itemId;
			}
			else {
				this.parentFolder.options[0].node = selectedNode;
			}
			
			var parentNodes = this._controller.reportTreeParser.getValidParents(this._selectedNodeId);		
			if (parentNodes != null && parentNodes.length > 0) {
				for (var i = 0; i < parentNodes.length; i++) {
					var parentNode = parentNodes[i];
					
					if (parentNode.report) {
						continue;
					}
					
					var parentNodeId = parentNode.itemId;
					
					if (parentNodeId == "" || parentNodeId == null) {
						continue;
					}
					
					if (selectedNode != null && parentNodeId == selectedNode.itemId) {
						continue;
					}
					
					if (parentNodeId != selectedNodeParentNodeId) {
						if (parentNode.shared && !this._controller.secMgr.canShare(parentNode)) {
							continue;
						}
					}
					
					var selected = (selectedNodeParentNodeId != null) && (selectedNodeParentNodeId == parentNodeId);
					var opt = new Option(parentNode.name, parentNodeId, selected, selected);
					opt.node = parentNode;
					this.parentFolder.options[this.parentFolder.options.length] = opt;
					
					if (selected) {
						opt.selected = true; // fix for IE6
					}
				}
			}
		}
		else {
			this.parentFolderDisplay.style.display = "none";
		}
	},
	
	_hideControls:function() {
	 	this.fileLabel.style.visibility = 'hidden';
		this.filePromptLabel.style.visibility = 'hidden';
		this._fileInputList.style.visibility = 'hidden';	
	},
	
	_createTeamAreaCombo: function() {
		var projectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea();
		this._parseTeamAreas(projectArea.children);
		
		if (this.teamAreaCombo.options.length > 0) {
			this.teamAreaCombo.options[0].selected = true;
		}
		else {
			this.teamAreaDisplay.style.display = "none";
		}
	},
	
	_parseTeamAreas: function(teamAreas) {	
		if (teamAreas == null || teamAreas.length == 0) {
			return;
		}
		for (var i = 0; i < teamAreas.length; i++) {
			var teamArea = teamAreas[i];
			
			var option = new Option(teamArea.name, teamArea.itemId, false, false);	
			this.teamAreaCombo.options[this.teamAreaCombo.options.length] = option;		
			this._parseTeamAreas(teamArea.children);
		}
	},
	
	_onGroupSelect : function() {	
		if(this._readOnly) {
			this.downloadButton.disabled = true;
			this.deleteButton.disabled = true;
			this.downloadButton._setStateClass();
			this.deleteButton._setStateClass();
			return;
		}
		var rows = this._table.getGroupSelectedRows();
		//enable download button only when a single row is selected or all the rows are selected
		var totalRows = this._reportDesignFilesTable.tBodies[0].getElementsByTagName("tr");
		
		if (totalRows.length == rows) {	
			this.downloadButton.disabled = false;
		} else if (rows == 1) {
			this.downloadButton.disabled = false;
		}
		else {
			this.downloadButton.disabled = true;
		}
		
		this.deleteButton.disabled = (rows <= 0);
		this.downloadButton._setStateClass();
		this.deleteButton._setStateClass();	
	},
	
	_fillOtherControls: function() {
		if (this._selectedReport != null) {
			this.nodeName.value = this._selectedReport.name;
			this.description.value = this._selectedReport.description;
		}
			
		this._setPageTitle(this.nodeName.value);
		
		if (this._resourceType == "REPORT_DESIGN") {
			this.typeDiv.appendChild(document.createTextNode(this._selectedReport.reportDesignType));
		}
		else {
			this.typeDisplay.style.display = "none";
		}
		this.nodeShared.checked = this._selectedReport.shared;
				
		//Disable controls if user is not owner
		var user = this._controller.secMgr.getUser();
		if (this._selectedReport != null && this._selectedReport.owner != null &&
				this._selectedReport.owner.itemId != user.itemId) {
			this._readOnly = true;
			this._disableControls();
		}
	},
	
	_disableControls: function() {
		this.saveButton.disabled = true;
		this.nodeName.disabled = true;
		this.description.disabled = true;
		this.parentFolder.disabled = true;
		this.typeDiv.disabled = true;
		this.nodeShared.disabled = true;
		this.reportDesignFileInput.disabled = true;		
	},
	
	_handleParentChange: function() {
		this._checkControlStates();
	},
	
	_checkControlStates: function() {
		if (this._selectedNode) {
			var shared = false;
					
			if (this.parentFolder.selectedIndex >= 0) {
				var opt = this.parentFolder.options[this.parentFolder.selectedIndex];
				if (opt.node) {
					shared = opt.node.shared;
				}
			}
			this.nodeShared.checked = shared;
		}
	},
	
	_updateSaveButtonState: function() {
		if(this._readOnly) {
			this.saveButton.disabled = true;
			return;
		}
		var disabled = true;
		if (this._nameChanged) {
			disabled = false;
		}
		else if (this._descriptionChanged) {
			disabled = false;
		}
		else if (this._parentFolderChanged){
			disabled = false;
		}
		else if (this._nodeSharedChanged) {
			disabled = false;
		}
		else if (this._reportDesignFilesChanged) {
			disabled = false;
		}
		else if (this._masterDesignFileChanged) {
			disabled = false;
		}
		else if (this.permissionsWidget && this.permissionsWidget._permissionsChanged) {
			disabled = false;
		}
		this.saveButton.disabled = disabled;
	},

	_handleSave: function() {
		if(this._readOnly) {
			return;
		}
		Util.removeChildren(this.messageArea);
		
		var name = this.nodeName.value;
		for(var i = 0; i < name.length; i++) {
			if (name.charCodeAt(i) == 47 || name.charCodeAt(i) == 92 || name.charCodeAt(i) == 42 || name.charCodeAt(i) == 63 ||
			    name.charCodeAt(i) == 124 || name.charCodeAt(i) == 62 || name.charCodeAt(i) == 60 || name.charCodeAt(i) == 58  || name.charCodeAt(i) == 34){
				this._displayErrorMessage(CommonMessages.ERROR_FILE_NAME+ ": * \\\ / | \  : < \" > ? ");
				return;
			}
		}
						
		if (this._resourceType == "REPORT_DESIGN") {
			var reportDesignFiles = 0;
			var reportDesignType = this._selectedReport.reportDesignType;
			if (reportDesignType == "BIRT") {
				var inputs = this._reportDesignFilesTable.tBodies[0].getElementsByTagName("input");
				for (var i = 0; i < inputs.length; i++) {
					var fileName = inputs[i].reportFileRowData.FileName;
					var fileParts = fileName.split(".");
					var fileExt = fileParts[fileParts.length - 1];
					if (fileExt.toLowerCase() == "rptdesign") {
						reportDesignFiles++;
					}
				}
			}
			else if (reportDesignType == "CRYSTAL_REPORTS") {
				for (var i = 0; i < inputs.length; i++) {
					var fileName = inputs[i].reportFileRowData.FileName;
					var fileParts = fileName.split(".");
					var fileExt = fileParts[fileParts.length - 1];
					if (fileExt.toLowerCase() == "rpt") {
						reportDesignFiles++;
					}
				}
			}
			
			if (reportDesignFiles == 0) {
				this._displayErrorMessage(ServiceabilityMessages.PROMPT_FOR_UPLOAD_REPORT_DESIGN_FILES);
				return;
			}
						
			var inputs = this._reportDesignFilesTable.tBodies[0].getElementsByTagName("input");
				
			if (this.fileListRowId < 1) {
				this._displayErrorMessage(ServiceabilityMessages.PROMPT_FOR_UPLOAD_REPORT_DESIGN_FILES);
				return;
			}
			
			this.masterReportChangedInput.value = this._masterDesignFileChanged;
			if (this.fileListRowId == 1) {
				var szRadioButton=inputs[0].reportFileRowData.Master;
				this.masterReportInput.value = szRadioButton.id;
			}
			
			var masterReportDesignSelection=false;
			
			if (this.fileListRowId > 1) {
				for (var i = 0; i < inputs.length; i++) {		
					if (inputs[i].type == 'radio'){
						var radioButton=inputs[i].reportFileRowData.Master;
						if (radioButton.checked) {
							this.masterReportInput.value = radioButton.id;
							masterReportDesignSelection = true;	
						}
					}
				}
				
				if (!masterReportDesignSelection) {
					this._displayErrorMessage(ServiceabilityMessages.PROMPT_FOR_MASTER_DESIGN_FILE);
					return;
				}
			}
			this.masterDesignFile = this.masterReportInput.value;
		}
		
		var name = this.nodeName.value;
		var description = this.description.value;
		var shared = this.nodeShared.checked;
		var parentFolderId = "";
		
		var projectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea();
		var projectAreaId = projectArea.itemId;
	
		// save the report design files 
		if (this._resourceType == "REPORT_DESIGN") {
			this._updateReportDesignFiles();
		}
				 
		if (this._selectedNodeId) {
			if (this.parentFolder.selectedIndex >= 0) {
				parentFolderId = this.parentFolder.options[this.parentFolder.selectedIndex].value;
			}
		}
		
		var owner = session.getAuthenticatedContributor();
		
		var self = this;
		
		var postExec = function() {
			self._controller.syncReportNodeChanged();					
			self._getSelectedNode();
			self._refreshSelectedReport();
			if (self._selectedNode)
				self._selectedNode.shared = self.nodeShared.checked;
			self._setPageTitle(self.nodeName.value);
			self._controller.reportTreeParser.expandFolder(parentFolderId);
		};
		
		this._postServiceExec = postExec;
		
		if (this._selectedNodeId) {
			var selectedNode = this._selectedNode;
			if (selectedNode) {
				var teamAreaId = (this._selectedReport.teamArea == null) ?
					"" : this._selectedReport.teamArea.itemId;
				this._errorMsg = dojo.string.substitute(ServiceabilityMessages.FAILED_TO_UPDATE_REPORT, [dojo.trim(name)]);
				
				var srh = new ServiceResponseHandler(this, "_updateReportNodeSuccess", "_remoteError");						
				var params = {id: selectedNode.itemId, reportItemId: selectedNode.report.itemId,
					name: name, parentFolderId: parentFolderId, projectAreaId: projectArea.itemId };
				ReportClient.postUpdateReportTreeNode(srh, params);
			}
		}
		else{
			this._updateReportNodeSuccess();
		}
		this.saveButton.disabled = true;
	},
	
	_updateReportDesignFiles:function() {
		this.createNode.action = TeamServerClient._getServicePrefix() + SERVICE_URI;
		this.reportDesignIdInput.value = this._selectedReport.itemId;
		
		// populate the list of server report design files that need to be removed
		var removeFilesLst = "";
		
		for (var i = 0; i < this._fileListArray.length; i++) {
	 		if (!this._fileListArray[i].isNew && this._fileListArray[i].isRemoved) {
				removeFilesLst = removeFilesLst+this._fileListArray[i].FileName+",";
			}
		}
		removeFilesLst = removeFilesLst.slice(0,(removeFilesLst.length-1)); // remove the extra delimiter
		
		this.filesToRemoveInput.value = removeFilesLst;
			
		var self = this;
		var count = 0;
		var headers = {};
		headers["content-type"] = "application/x-www-form-urlencoded; charset=utf-8";
		var bindArgs = {
			form: document.getElementById("createNode"),
			handleAs: "text/html",
			headers: headers,			
			handle: function(type, data, evt) {
				console.debug("Type:", type);
				console.debug("Response:", data);
				Util.removeChildren(self.messageArea);
			}
		};
		
		var request = dojo.io.iframe.send(bindArgs);
	},
	
	_updateReportNodeSuccess: function() {
		var name = this.nodeName.value;
		if (name != this._selectedReport.name || this.description.value != this._selectedReport.description || this._selectedReport.shared != this.nodeShared.checked) {
			var srh = new ServiceResponseHandler(this, "_success", "_remoteError");
			var teamAreaId = (this._selectedReport.teamArea == null) ?	"" : this._selectedReport.teamArea.itemId;			
			var params = {id: this._selectedReport.itemId, name: name, 
				description: this.description.value, teamAreaId: teamAreaId, shared: this.nodeShared.checked };
			
			if (this._resourceType == "REPORT_DESIGN") {
				this._errorMsg = dojo.string.substitute(ServiceabilityMessages.FAILED_TO_UPDATE_REPORT_DESIGN, [dojo.trim(name)]);
				ReportClient.postUpdateReportDesign(srh, params);								
			}
			
			else if (this._resourceType == "REPORT_QUERY") {
				this._errorMsg = dojo.string.substitute(ServiceabilityMessages.FAILED_TO_UPDATE_REPORT_QUERY, [dojo.trim(name)]);
				ReportClient.postUpdateReportQuery(srh, params);
			}
			
			else if (this._resourceType == "REPORT_SNAPSHOT") {
				this._errorMsg = dojo.string.substitute(ServiceabilityMessages.FAILED_TO_UPDATE_REPORT_SNAPSHOT, [dojo.trim(name)]);
				ReportClient.postUpdateReportSnapshot(srh, params);
			}
		}
		else {
			this._success();
		}	
	},
	
	// get report design files from the server and initilaize controls
	_getReportDesignFiles: function() {
		var szURL = TeamServerClient._getServicePrefix()+SERVICE_URI+"getReportDesignFiles?reportdesignId="+this._selectedReport.itemId;
	
		//TODO: Need to replace the following asynchronous call to synchronous call
		
		var dfd = dojo.xhrGet({ url: szURL,handleAs: "json"});
    	return dfd;
	},
	
	reportDesignFilesHandler:function(data) {
    	var szReportFiles = data[0].FileNames.split(",");
	
		for (var i = 0; i < szReportFiles.length; i++) {
			var masterrB;
			
			if (szReportFiles[i] == data[1].MasterFile) {
				if (dojo.isIE) {
					masterrB = document.createElement('<input name=" " type="radio" checked>');
				}
				else if (dojo.isMozilla) {
					masterrB = document.createElement('input');
					masterrB.type = 'radio';
					masterrB.checked = true;					
				}
				this.masterDesignFile = szReportFiles[i];
			}
			else {
				if (dojo.isIE) {
					masterrB = document.createElement('<input name=" ">' );
				} else if(dojo.isMozilla) {
					masterrB = document.createElement('input');
				}
				masterrB.type='radio';
	    	}
			masterrB.name = "MasterFile";
			masterrB.id= szReportFiles[i];
			
			var selectcB = document.createElement('input');
        	selectcB.type = 'checkbox';
        	selectcB.id = szReportFiles[i];
	
			var reportFileRowData = {
				rowId: this.fileListRowId,
				itemId: this.fileListRowId,
				Select: selectcB,
				FileName: szReportFiles[i],
				Description: this._selectedReport.itemId,
				Master: masterrB,
				Toolbar:null,
				filePath: "server", //server copy
				isNew: false,
				isRemoved: false,
				inputFile: null  //reference to input file
			};
			selectcB.reportFileRowData = reportFileRowData;
			masterrB.reportFileRowData = reportFileRowData;
			this._table.store.addData(reportFileRowData,0);
			
			// increment table row ID.
			this.fileListRowId++;  
			
			// push the row data into table
			this._fileListArray.push(reportFileRowData);
		}
	},
	
	_addFileInput: function() {
		var fullPath = this.currentInput.value.replace(/\\/g, "\/");
		
		var parts = fullPath.split("\/");
		var fileName = parts[parts.length - 1];
		
		this.currentInput.style.position = 'absolute';
		this.currentInput.style.left = '-1000px';
		
		var szCurrentInput = this.currentInput;
		
		this.currentInput = document.createElement("INPUT");
		this.currentInput.type = "file";
		this.currentInput.name = 'file_' + this.count++;
		this.currentInput.size = "80";
		var self = this;
		
		this.currentInput.dojoattachpoint = this.currentInput.name;
				
		this._fileInputList.appendChild(this.currentInput);
				
		// check for duplicate report design files
		for (var i = 0; i < this._fileListArray.length; i++)  {
			if (fileName == this._fileListArray[i].FileName) {
				if (!this._fileListArray[i].isNew && this._fileListArray[i].isRemoved) {  //if server copy and not remo
					continue;
				}
				else {
					this._displayErrorMessage(ServiceabilityMessages.REPORT_DESIGN_ALREADY_EXISTS);
					dojo.connect(this.currentInput, "onchange", this, "_addFileInput");	
					return;
				}
			}
		}
		
		var masterrB;
		
	    if (dojo.isIE) {
			masterrB = document.createElement('<input name=" ">');
		} else if (dojo.isMozilla) {
			masterrB = document.createElement('input');
		}
    	masterrB.type = 'radio';
    	masterrB.id = fileName;
		masterrB.name = "MasterFile";
		
		var selectcB = document.createElement('input');
    	selectcB.type='checkbox';
    	selectcB.id= fileName;
		
		var reportFileRowData = {
			rowId: this.fileListRowId,
			itemId: this.fileListRowId,
			Select: selectcB,
			FileName: fileName,
			Description: this._selectedReport.itemId,
			Master: masterrB,
			Toolbar:null,
			filePath: fullPath, // local copy
			isNew: true,
			isRemoved: false,
			inputFile: szCurrentInput  // reference to input file
		};
		
		selectcB.reportFileRowData = reportFileRowData;
		masterrB.reportFileRowData = reportFileRowData;
		
		// increment table row ID.
		this.fileListRowId++;  
		
		// push the row data into table
		this._fileListArray.push(reportFileRowData);
		
		this._table.store.addData(reportFileRowData,i);
		
		dojo.connect(this.currentInput, "onchange", this, "_addFileInput");
		this._reportDesignFilesChanged = true;
		this._updateSaveButtonState();
	},
	
	_downloadReportDesignFiles:function() {
		var inputs = this._reportDesignFilesTable.tBodies[0].getElementsByTagName("input");
		var szSelectedRows = this._table.getGroupSelectedRows();
		var totalRows = this._reportDesignFilesTable.tBodies[0].getElementsByTagName("tr");
		
		if (totalRows.length == szSelectedRows) {	
			var DOWNLOAD_SERVICE_URI = TeamServerClient._getServicePrefix()+"com.ibm.rational.report.common.IReportContentRestService/downloadreport/"+this._selectedReport.itemId;
			location.href = DOWNLOAD_SERVICE_URI;
		}
		else {
			for (var i = 0; i < inputs.length; i++) {
				if (inputs[i].checked && inputs[i].type=='checkbox') {
					var DOWNLOAD_SERVICE_URI =TeamServerClient._getServicePrefix()+SERVICE_URI+"downloadReportFile?reportdesignId="+this._selectedReport.itemId+"&rptFileName="+inputs[i].reportFileRowData.FileName;
					location.href = DOWNLOAD_SERVICE_URI;
				}
			}
		}
	},
	
	_deleteReportDesignFiles:function(){
		var inputs = this._reportDesignFilesTable.tBodies[0].getElementsByTagName("input");
		
		for (var i = 0; i < inputs.length; i++) {
			if (inputs[i].checked && inputs[i].type=='checkbox') {
				this._performRemove(inputs[i].reportFileRowData);
			}
		}
		this._refreshTable();
		this._reportDesignFilesChanged = true;
		this._updateSaveButtonState();		
	},
	
	_performRemove: function(reportFileRowData) {
		for (var i = 0; i < this._fileListArray.length; i++) {
	 		if (this._fileListArray[i].rowId == reportFileRowData.rowId) {
	 			if (reportFileRowData.isNew) {
 					// decrement table row counter and the file counter
 					this.count--;
 					
	 				// remove the row from the array
	 				this._fileListArray.splice(i, 1);
	 				
	 				// remove the "input" file div element from the form
	 				this._fileInputList.removeChild(reportFileRowData.inputFile);
	 			}
	 			else{
	 				reportFileRowData.isRemoved=true;
	 			}
				
	 			this.fileListRowId--;
				break;
			}
 		}
	},
	
	_refreshTable: function() {
		// remove all current rows
		this._table.store.clearData();
		
		// recreate the table	
		for (var i = 0; i < this._fileListArray.length; i++) {
			if (this._fileListArray[i].isRemoved) {
				continue;
			}
			else {
				this._table.store.addData(this._fileListArray[i],i);
			}
		}
		this.downloadButton.disabled = true;
		this.deleteButton.disabled = true;
		this.downloadButton._setStateClass();
		this.deleteButton._setStateClass();	
	},	
	
	_success: function() {
		Util.removeChildren(this.messageArea);
		if (this._postServiceExec != null) {
			this._postServiceExec();
		}
		if (this.permissionsWidget != null) {
			this.permissionsWidget._updateAclEntries();
		}
	},
	
	_remoteError: function(error) {
		this._msgFormatter.displayErrorMessage(this._errorMsg, error);
	},
	
	_displayErrorMessage: function(errorMsg) {
		var errorDialog = new com.ibm.rational.report.web.core.app.internal.common.ErrorDialog();
		errorDialog.displayError(errorMsg);
	},
	
	_newTabSelected: function(tab) {
		if (tab.dojoAttachPoint == "permissionContentPane") {
			if (this.permissionsWidget == null  && this._createPermissionsTab == true) {
				this.permissionsWidget = new PermissionsWidget({
					repositoryItemId: this.repositoryItemId,
					messageArea: this.messageArea,
					resourceType: this._resourceType,
					parentWidget: this
				});
				this.permissionContentPane.appendChild(this.permissionsWidget.domNode);
			}
		}
		else if (tab.dojoAttachPoint == "viewReportContentPane") {
			if (this.reportContent == null) {
				this.reportContent = new ReportContent({
					reportItemId: this.repositoryItemId,
					design: null,
					reportParamNames: null,
					reportParamValues: null,
					collapseParameters: false,
					controller: this._controller,
					type: this._resourceType,
					parentWidget: this
				});
				this.viewReportContentPane.appendChild(this.reportContent.domNode);
			}
		}	
	}
	
});

})();